--[[
Name: LibTalentQuery-1.0
Revision: $Rev: 64729 $
Author: Rich Martel (richmartel@gmail.com)
Documentation: http://wowace.com/wiki/LibTalentQuery-1.0
SVN: http://svn.wowace.com/wowace/trunk/LibTalentQuery-1.0
Description: Library to help with querying unit talents
Dependancies: LibStub, CallbackHandler-1.0
License: LGPL v2.1

Example Usage:
	local TalentQuery = LibStub:GetLibrary("LibTalentQuery-1.0")
	TalentQuery.RegisterCallback(self, "TalentQuery_Ready")
	local spec = {}
	...
	TalentQuery:Query(unit)
	...
	function MyAddon:TalentQuery_Ready(e, unit, isName)
		local unitname
		if isName then
			unitname = unit
		else
			unitname = TalentQuery:UnitNameServer(unit)
		end
		for tab = 1, GetNumTalentTabs(true) do
			local treename, _, pointsspent = GetTalentTabInfo(tab, true)
			if (pointsspent > 30) then
				spec[unitname] = treename
			end
		end
		...
	end
]]

local INSPECTDELAY = 1
local MAJOR, MINOR = "LibTalentQuery-1.0", "$Rev: 64729 $"

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

lib.events = lib.events or LibStub("CallbackHandler-1.0"):New(lib)

lib.frame = lib.frame or CreateFrame("Frame", MAJOR .. "_Frame")
lib.frame:SetScript("OnEvent", function(this, event, ...) lib[event](lib, ...) end)
lib.frame:SetScript("OnUpdate", function() return lib:OnUpdate() end)

local inspectingUnit = nil
local inspectingUnitIsName = nil
local inspectQueue = {}
local nextInspect = nil

function lib:OnUpdate()
	if nextInspect and (nextInspect < GetTime()) then
		self:CheckInspectQueue()
	end
end

hooksecurefunc("NotifyInspect", function(...) return lib:NotifyInspect(...) end)
function lib:NotifyInspect(unit)
	-- If this library did not call NotifyInspect, and we are waiting for a query, then
	-- stop waiting, and put the inspectingUnit back on the queue.
	if inspectingUnit and (inspectingUnit ~= unit) then
		self.frame:UnregisterEvent("INSPECT_TALENT_READY")
		if inspectingUnitIsName then
			inspectQueue[inspectingUnit] = "name"
		else
			inspectQueue[inspectingUnit] = true
		end
		inspectingUnit = nil
		inspectingUnitIsName = nil
		nextInspect = GetTime() + INSPECTDELAY
	end
end

function lib:INSPECT_TALENT_READY()
	local unit, isName
	if inspectingUnit then
		self.frame:UnregisterEvent("INSPECT_TALENT_READY")
		unit = inspectingUnit
		isName = inspectingUnitIsName
		inspectingUnit = nil
		inspectingUnitIsName = nil
	end
	self.events:Fire("TalentQuery_Ready", unit, isName)
end

function lib:UnitNameServer(unit)
	local name, server = UnitName(unit)
	if name and server then
		name = name .. "-" .. server
	end
	return name
end

function lib:UnitId(name)
	local unit
	if (not unit) and (GetNumRaidMembers() > 0) then
		for i = 1, GetNumRaidMembers() do
			if UnitExists("raid" .. i) and (name == self:UnitNameServer("raid" .. i)) then
				unit = "raid" .. i
				break
			end
		end
	end
	if (not unit) and (GetNumPartyMembers() > 0) then
		for i = 1, GetNumPartyMembers() do
			if UnitExists("party" .. i) and (name == self:UnitNameServer("party" .. i)) then
				unit = "party" .. i
				break
			end
		end
	end
	if (not unit) and UnitExists("player") and (name == self:UnitNameServer("player")) then
		unit = "player"
	end
	if (not unit) and UnitExists("focus") and (name == self:UnitNameServer("focus")) then
		unit = "focus"
	end
	if (not unit) and UnitExists("target") and (name == self:UnitNameServer("target")) then
		unit = "target"
	end
	if (not unit) and (GetNumRaidMembers() > 0) then
		for i = 1, GetNumRaidMembers() do
			if UnitExists("raid" .. i .. "target") and (name == self:UnitNameServer("raid" .. i .. "target")) then
				unit = "raid" .. i .. "target"
				break
			end
		end
	end
	if (not unit) and (GetNumPartyMembers() > 0) then
		for i = 1, GetNumPartyMembers() do
			if UnitExists("party" .. i .. "target") and (name == self:UnitNameServer("party" .. i .. "target")) then
				unit = "party" .. i .. "target"
				break
			end
		end
	end
	if (not unit) and UnitExists("pettarget") and (name == self:UnitNameServer("pettarget")) then
		unit = "pettarget"
	end
	if (not unit) and (GetNumRaidMembers() > 0) then
		for i = 1, GetNumRaidMembers() do
			if UnitExists("raidpet" .. i .. "target") and (name == self:UnitNameServer("raidpet" .. i .. "target")) then
				unit = "raidpet" .. i .. "target"
				break
			end
		end
	end
	if (not unit) and (GetNumPartyMembers() > 0) then
		for i = 1, GetNumPartyMembers() do
			if UnitExists("partypet" .. i .. "target") and (name == self:UnitNameServer("partypet" .. i .. "target")) then
				unit = "partypet" .. i .. "target"
				break
			end
		end
	end
	if (not unit) and UnitExists("targettarget") and (name == self:UnitNameServer("targettarget")) then
		unit = "targettarget"
	end
	return unit
end

function lib:CheckInspectQueue()
	local repeatqueuecheck
	if (not inspectingUnit) then
		for unit, unittype in pairs(inspectQueue) do
			if unittype == "name" then
				local uid = self:UnitId(unit)
				if uid then
					unit = uid
				else
					inspectQueue[unit] = nil
					unit = nil
				end
			end
			if unit then
				repeatqueuecheck = true
				if UnitIsVisible(unit) and UnitIsConnected(unit) then
					if (string.find(unit, "player") or string.find(unit, "party") or string.find(unit, "raid")) and (not string.find(unit, "target")) and (not string.find(unit, "pet")) then
						inspectingUnit = unit
						inspectingUnitIsName = nil
					else
						name = self:UnitNameServer(unit)
						if name then
							inspectingUnit = name
							inspectingUnitIsName = true
							inspectQueue[unit] = nil
						end
					end
					inspectQueue[inspectingUnit] = nil
					self.frame:RegisterEvent("INSPECT_TALENT_READY")
					NotifyInspect(inspectingUnit)
					break
				end
			end
		end
	else
		repeatqueuecheck = true
	end
	if repeatqueuecheck then
		nextInspect = GetTime() + INSPECTDELAY
	else
		nextInspect = nil
	end
end

function lib:Query(unit)
	if unit and UnitExists(unit) and UnitIsPlayer(unit) then
		if (string.find(unit, "player") or string.find(unit, "party") or string.find(unit, "raid")) and (not string.find(unit, "target")) and (not string.find(unit, "pet")) then
			inspectQueue[unit] = true
			nextInspect = GetTime() + INSPECTDELAY
		else
			name = self:UnitNameServer(unit)
			if name then
				inspectQueue[name] = "name"
				nextInspect = GetTime() + INSPECTDELAY
			end
		end
	else
		error("Usage: lib:Query(unit): 'unit' must be a valid player unit.")
	end
end
